package com.altinity.opsview.odw.bc;

/**
 * Copyright (C) 2008 Opsera Limited. All rights reserved
 *
 * This file is part of Opsview
 * 
 * Opsview is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * Opsview is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Opsview; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301  USA
 * 
 */

import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;

import org.dom4j.Element;

import com.altinity.opsview.odw.CommandLineParams;
import com.altinity.opsview.odw.elements.AvailabilityOverview;
import com.altinity.opsview.odw.elements.AvailabilityTrendChart;
import com.altinity.opsview.odw.elements.AverageAvailabilityChart;
import com.altinity.opsview.odw.elements.AverageAvailibilityForHostGroupsChart;
import com.altinity.opsview.odw.elements.HostGroupAvailability;
import com.altinity.opsview.odw.elements.HostGroupExceptionDetail;
import com.altinity.opsview.odw.exception.InvalidUsageException;
import com.altinity.opsview.odw.exception.ReportException;
import com.excira.utils.DateUtils;
import com.excira.utils.StringUtils;

public class AvailabilityReportBC extends BaseReportBC {
	
	/**
	 * 
	 */
	public void createReport() throws ReportException {
		
		// First set up each module and make sure we have the data
		// to generate report
		AvailabilityOverview availOverview =
			new AvailabilityOverview(reportCriteria, dao, configDao, log);
		availOverview.runQuery();
		
		// Daily availability
		AverageAvailabilityChart availChart = 
			new AverageAvailabilityChart(reportCriteria, dao, configDao, log);
		availChart.runQuery();
		
		// Availability trend
		AvailabilityTrendChart trendChart = 
			new AvailabilityTrendChart(reportCriteria, dao, configDao, log);
		trendChart.runQuery();
		
		// Host group availability summary
		HostGroupAvailability hgAvail =
			new HostGroupAvailability(reportCriteria, dao, configDao, log);
		hgAvail.runQuery();
		
		// Host group summary
		AverageAvailibilityForHostGroupsChart hgAvailChart =
			new AverageAvailibilityForHostGroupsChart(reportCriteria, dao, configDao, log);
		hgAvailChart.runQuery();
		
		// Exception detail
		HostGroupExceptionDetail exceptionDetail =
			new HostGroupExceptionDetail(reportCriteria,dao, configDao, log);
		exceptionDetail.runQuery();
		
		// If data is valid for each module, add modules
		// to report
		Element reportSections = getReportSections();
		
		//section Availability Summary
		Element section = reportSections.addElement("section");
		section.addElement("title").setText("Availability Summary");
		section.addAttribute("name", "availabilitySummary");
		Element elements = section.addElement("elements");
		
		availOverview.addElement(elements);
		availChart.addElement(elements);	
		trendChart.addElement(elements);
		
		// Section: Host Group Availability
		section = reportSections.addElement("section");
		section.addElement("title").setText("Hostgroup Availability");
		section.addAttribute("name", "hostgroupAvailabilitySummary");
		elements = section.addElement("elements");
		
		hgAvail.addElement(elements);
		hgAvailChart.addElement(elements);
		exceptionDetail.addElement(elements);
		
		// If there are no errors at this point, the report has 
		// been genrated and the host group availability is 
		// the last section
		if ( getLog().isClean() )
			section.addAttribute("lastSection", "true");
	}

	/**
	 * 
	 */
	protected Date initStartDate(CommandLineParams params) throws InvalidUsageException {
		
		String dateParam = params.getStrStartDate();
		
		// First confirm that start date is valid. Must be yyyy-MM or -n
		if ( !DateUtils.isValidDate(dateParam, CommandLineParams.YYYY_MM_REGX, "yyyy-MM")
				&& !StringUtils.matchesPattern(dateParam, CommandLineParams.N_REGX) )
			throw new InvalidUsageException();
		
		// First get the current calendar
		Calendar cal = Calendar.getInstance();
		
		// First check if a start date has been specified
		if ( StringUtils.notNullOrEmpty(dateParam) ) {
			 
			// Handle case where -n has been specified
			if ( StringUtils.matchesPattern(dateParam, "[0]|-\\d*") ) {
				
				// Update month field of calendar and get start date
				cal.add(Calendar.MONTH, new Integer(dateParam));
				return DateUtils.parse(cal.get(Calendar.YEAR) + "-" + (cal.get(Calendar.MONTH) + 1), "yyyy-MM");
			}
			// Otherwise yyyy-MM has been specified
			else {
			
				// Confirm the specified month is this month or earlier
				Calendar specifiedCal = Calendar.getInstance();
				specifiedCal.setTime(DateUtils.parse(dateParam, "yyyy-MM"));
				
				return DateUtils.parse(dateParam, "yyyy-MM");
			}
		}
		// Otherwise no date was specified so use the current month
		else {
			
			int month = cal.get(Calendar.MONTH) + 1; // Add 1 as months start from 0
			int year = cal.get(Calendar.YEAR);
			return DateUtils.parse(year + "-" + month, "yyyy-MM");
		}
	}
	
	/**
	 * 
	 */
	protected Date initEndDate(CommandLineParams params) {
		
		Calendar startCal = Calendar.getInstance();
		startCal.setTime(startDate);
		
		// The end date is always last second of the 
		// last day in the start month
		StringBuffer strEndDate = new StringBuffer();
		strEndDate.append(startCal.get(Calendar.YEAR));
		strEndDate.append("-");
		strEndDate.append((startCal.get(Calendar.MONTH) + 1));
		strEndDate.append("-");
		strEndDate.append(startCal.getActualMaximum(Calendar.DAY_OF_MONTH));
		strEndDate.append(" ");
		strEndDate.append("23:59:59");
		return DateUtils.parse(strEndDate.toString(), "yyyy-M-dd HH:mm:ss");
	}
	
	
	/**
	 * 
	 */
	public String getOutputFilePath() {
		
		Date dataStartDate = reportCriteria.getReportStartDate();
		String currentDir = System.getProperty("user.dir");
		
		StringBuffer filePath = new StringBuffer();
		filePath.append(currentDir + "/reports/");
		filePath.append("generic_availability-");
		filePath.append(reportCriteria.getReportSelection().replace(' ', '_'));
		filePath.append("-");
		SimpleDateFormat format = new SimpleDateFormat("MMMM-yyyy");
		filePath.append(format.format(dataStartDate)); 
		filePath.append("-");
		format.applyPattern("yyyy-MM-dd");
		filePath.append(format.format(Calendar.getInstance().getTime()));
		filePath.append(".pdf");
		
		return filePath.toString();
	}	
}
